﻿param(
[Parameter(Mandatory=$True,
           helpmessage="Unique name to ensure data is counted only once and result can be identifed")]
           [string]$namespace,

[Parameter(Mandatory=$True,
           helpmessage="Enter directory where the IBM Spectrum Protect Snapshot exchange instance is located")]
           [string]$fcminstance, 

[Parameter(Mandatory=$True,
           helpmessage="Enter directory where the output should be written to")]
           [string]$directory,
           
[Parameter(Mandatory=$False,
           helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot Exchange options file")]
           [string]
           $tsmoptfile,

[Parameter(Mandatory=$False,
           helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot Exchange configurations file")]
           [string]
           $configfile,

[Parameter(Mandatory=$False,
           helpmessage="Enter the BE Factor 0-1")]
           [decimal]$befactor,

[Parameter(Mandatory=$False,
           helpmessage="Enter directory where Remote Exchange script is located")]
           [string]$exchangedir            
)

$productId   = "13"
$productName = "IBM Spectrum Protect Snapshot for Microsoft Exchange"   
$itemTtype   = "fcm";

$global:offload = 'n'
$global:offloadFlag = $False
$global:countBE = 0

function connect()
{
   $psVersion = Invoke-Command -Command {$PSVersionTable.PSVersion}

   if ($psVersion.Major -ge 3)
   {
      $ErrorActionPreference = "SilentlyContinue"
      if(!(Get-Command Get-MailboxDatabaseCopyStatus))
      {  
         $ErrorActionPreference = "Continue"
         Write-Host "Connecting to Exchange Server...`n"    
         Invoke-Expression -Command ". '$exchangedir\RemoteExchange.ps1'"
         Invoke-Expression -Command "Connect-ExchangeServer -auto"
      }
   }
   else
   {
      Write-Host "The PowerShell version is not compatible with IBM Spectrum Protect Snapshot`n"
      exit
   }
}

function fcmFE()
{
   Write-Host "`nQuery protected Mailbox Databases"         
   $cmdOut = Get-MailboxDatabaseCopyStatus | where {$_.ActiveCopy -eq $true } | select name   

   if ($? -ne $True)
   {      
      exit
   }   
          
   return $cmdOut  
}

function parseUsageData($cmdOutFE, $cmdOutLU, $cmdOutBE)
{
   $splitLine = $False
   $dbNames = @()
   $dbSizeFE = 0
   $dbCount = 0
   $dbSizeBE = 0
   $dbSizeLU = 0
   $countLU = 0
   
   
   if($befactor -le 0 -or $befactor -ge 1)
   {
      $befactor = 1;
   }
   
   foreach ($dbFE in $cmdOutFE)
   {
      $tokens = [regex]::Split($dbFE.name, "\\")
      $dbNames += $tokens[0]
   }

   $cmdOutFE = Get-MailboxDatabase -status | where {$_.Recovery -eq $false } | select name,databasesize,last*

   foreach ($dbFE in $cmdOutFE)
   {
      if ($dbNames -contains $dbFE.name)
      {
         Write-Host "`nGetting front end size for" $dbFE.name ":" $dbFE.databasesize
         $sizeStr = $dbFE.databasesize -replace ".+\((.+)\s\w+\)", '$1'
         $size = $sizeStr -replace ",", ""         
         $dbSizeFE += $size
         $dbCount +=1         
      }      
   }
   
   [int]$dbSizeFE = [decimal]::round($dbSizeFE/1024/1024)

   foreach ($dbLU in $cmdOutLU)
   {
     [int]$sizeLU = [decimal]::round($dbLU.ManagedCapacity/1024/1024)
     $dbSizeLU += $sizeLU
     $countLU++
   }
   
   foreach ($db in $cmdOutFE)
   {
      if ($cmdOutBE -contains $db.name)
      {
         $sizeStr = $db.databasesize -replace ".+\((.+)\s\w+\)", '$1'
         $sizeBE = $sizeStr -replace ",", "" 
         $sizeBE = ($sizeBE/1024/1024) * $countBE * $befactor
         Write-Host "`nGetting back end size for" $db.name ":" $sizeBE "(MB)"       
         $dbSizeBE += $sizeBE    
      }      
   }
   [int]$dbSizeBE = [decimal]::round($dbSizeBE)

   Write-Host "`nIBM Spectrum Protect Snapshot FE"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-Host "Number of objects:  $dbCount"
   Write-Host "Exchange (fcm fe):  $dbSizeFE (MB)"

   Write-Host "`nIBM Spectrum Protect Snapshot LUN"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-Host "Number of Objects:  $countLU"
   Write-Host "Exchange (fcm lun): $dbSizeLU (MB)" 

   Write-Host "`nIBM Spectrum Protect Snapshot BE"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-Host "Snapshots:          $countBE" 
   Write-Host "Exchange (fcm be):  $dbSizeBE (MB)`n" 

   if ($offloadFlag)
   {
      Write-Host "Backups to IBM Spectrum Protect Server detected: Yes!`n"
   }
   else
   {
      Write-Host "Backups to IBM Spectrum Protect Server detected: No`n"
   }
   

   createXmlFile $dbCount $dbSizeFE $countBE $dbSizeBE $countLU $dbSizeLU $offload
      
}

function fcmLU()
{
   $cmdStrLU = $fcminstance.Substring(0,$fcminstance.Length)
   $cmdStrLU = $cmdStrLU + "\FmModuleExc.dll"
   
   Import-Module $cmdStrLU
   
   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOutLU = Invoke-Command -Command{Get-DpExcManagedCapacity -detail -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOutLU = Invoke-Command -Command{Get-DpExcManagedCapacity -detail}
   }

   return $cmdOutLU
}

function foo { $global:someGlobal = 42; $someGlobal }

function fcmBE()
{
   $dbNames = @()
   $cmdOutBe = @()
   $cmdOutExc = Get-MailboxDatabaseCopyStatus | where {$_.ContentIndexState -eq 'Healthy'} |select name 

   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOutBackup = Invoke-Command -Command{Get-DpExcBackup -all -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOutBackup = Invoke-Command -Command{Get-DpExcBackup -all}
   }

   foreach ($db in $cmdOutExc)
   {
      $tokens = [regex]::Split($db.name, "\\")
      $dbNames += $tokens[0]
   }

   $cmdOutBE = $dbNames

   foreach ($dbBE in $cmdOutBackup)
   {
       if ($dbBE.BackupLocation.equals('Loc'))
       {
          $global:countBE++
       }
   }

   foreach ($backupLocation in $cmdOutBackup)
   {
       if ($backupLocation.BackupLocation.equals('Srv'))
       {
          $global:offload = 'y'
          $global:offloadFlag = $True
          break;
       }
   }

   Remove-Module -Name FmModuleExc
   
   return $cmdOutBE
}

function createXmlFile($dbCount, $dbSizeFE, $countBE, $dbSizeBE, $countLU, $dbSizeLU, $offload)
{ 
   $cmdStr = '.\dsmfecc "--create" "--namespace=$namespace" "--productid=$productId" "--directory=$directory" "--applicationentity=$namespace" "--numberofobjects=$dbCount" "--size=$dbSizeFE" "--type=$itemTtype" "--fcmbenumberofobjects=$countBE" "--fcmbesize=$dbSizeBE" "--fcmlunnumberofobjects=$countLU" "--fcmlunsize=$dbSizeLU" "--offload=$offload" 2>&1'
   
   Write-Host "`nCreate XML file"
      
   Invoke-Expression $cmdStr
   if ($LastExitCode -ne 0)
   {
      Write-Host "Create XML file failed."      
      exit
   }    
}

Write-Host "`n"
Write-Host "********************************************************************"
Write-Host "********      IBM Spectrum Protect Suite - Front End       *********"
Write-Host "********          Terabyte (TB) Capacity Report            *********"
Write-Host "********************************************************************`n"
Write-Host "$productName`n"

connect
$queryFE = fcmFE
$queryLU = fcmLU
$queryBE = fcmBE
parseUsageData $queryFE $queryLU $queryBE


