﻿Param(
    [Parameter(mandatory=$true,               
               helpmessage="Enter the name of the SQL Server instance to use")]
               [string]               
               $applicationentity,

    [Parameter(mandatory=$true,               
               helpmessage="Unique name to ensure data is counted only once and result can be identifed")]
               [string]               
               $namespace,

    [Parameter(Mandatory=$True,
               helpmessage="Enter directory where IBM Spectrum Protect Snapshot is installed")]
               [string]
               $fcminstance, 

    [Parameter(Mandatory=$True,
               helpmessage="Enter directory where the output should be written to")]
               [string]
               $directory,

    [Parameter(Mandatory=$False,
               helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot SQL options file")]
               [string]
               $tsmoptfile,

    [Parameter(Mandatory=$False,
               helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot SQL configurations file")]
               [string]
               $configfile,

    [Parameter(Mandatory=$False,
               helpmessage="Enter the BE Factor 0-1")]
               [decimal]$befactor
)  

[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.Smo") | Out-Null
[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.SqlEnum") | Out-Null
[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.SmoEnum") | Out-Null
[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.ConnectionInfo") | Out-Null

# Modification Date: 05/14/15

$productId   = "14"
$productName = "IBM Spectrum Protect Snapshot for Microsoft SQL"   
$itemTtype   = "fcm";

function Show-FcmSqlReport
{  
   process 
   {
      $totalUsedDBSizeMB = 0
      $dbCount           = 0

      $dbSizeMBBE        = 0
      $totalDbCountBE    = 0

      $lunSizeMB         = 0
      $totalLunSizeMB    = 0
      $lunCount          = 0

      $offload           = 'n'

      if($befactor -le 0 -or $befactor -ge 1)
      {
      $befactor = 1;
      }

      try
      {
         $cmdStr = $fcminstance.Substring(0,$fcminstance.Length)
         $cmdStr  = $cmdStr + "\FmModuleSql.dll"
         Import-Module $cmdStr
      }
      catch
      {
         Write-Host $_.Exception
      }
   
      try
      {       
         $dbSizeAllocations = Get-FcmSqlUsedDbSize -sqlInstance $applicationentity
         $dbSizeAllocations | select ComputerName,InstanceName,DbName,AAGName,UsedSizeKb,LastBackupDate | ft -AutoSize
         $dbCount           = $dbSizeAllocations.count

         $totalUsedDBSizeMB= ($dbSizeAllocations | Measure-Object -Property UsedSizeKb -Sum).sum / 1024  
         $totalUsedDBSizeMB = [decimal]::round($totalUsedDBSizeMB)


         $FcmSqlDBBackups = Get-FcmSqlBackups
         foreach ($db in $dbSizeAllocations)
         {
           $dbName          = $db.DbName
           $aagName         = $db.AAGName
           $sqlInstanceName = $db.InstanceName
           $dbBackupCount   = 0

           if ($aagName)
           {
              $dbBackupInfo    = $FcmSqlDBBackups | Where-Object {($_.SQLServerName -eq $aagName) -and
                                                    ($_.SQLDatabaseName -eq $dbName) -and ($_.BackupLocation -eq "Loc")}
           }
           else
           {
              $dbBackupInfo    = $FcmSqlDBBackups | Where-Object {($_.SQLServerName -eq $sqlInstanceName) -and
                                                    ($_.SQLDatabaseName -eq $dbName) -and ($_.BackupLocation -eq "Loc")}
           }

           $dbBackupCount   = $dbBackupInfo.count
           $totalDbCountBE += $dbBackupCount

           $totalBkupSize   = $db.UsedSizeKb * $dbBackupCount * $befactor / 1024
           $dbSizeMBBE     += $totalBkupSize


           Write-Host "Found $dbBackupCount Local backups for SQL Database '$dbName'"

           $dbBackupInfo    = $FcmSqlDBBackups | Where-Object {$_.BackupLocation -eq 'Srv'}
           $dbBackupCount   = $dbBackupInfo.count

           if ($dbBackupCount -gt 0)
           {
              $offload = 'y'
           }

         }

         if ($offload -eq 'y')
         {
              Write-Host "`nDetected backups sent to IBM Spectrum Protect Server"
         }
         
         
         Write-Host "`nIBM Spectrum Protect Snapshot for Microsoft SQL Server"
         Write-Host "-----------------------------------------------------------"
         Write-Host "Query result"
         Write-Host "Number of SQL Databases:   $dbCount"
         Write-Host "Front End Total Size:      $totalUsedDBSizeMB (MB)`n"
   

         $dbSizeMBBE     = [decimal]::round($dbSizeMBBE)

         Write-Host "Number of backups:         $totalDbCountBE"
         Write-Host "Back End Total Size :      $dbSizeMBBE (MB)`n"

      }
      catch
      {
         Write-Host $_.Exception
      }

      try
      {
         $LunManagedCapacity = Get-FcmSqlManagedCapacity
         
         foreach ($lun in $LunManagedCapacity)
         {
            [int]$lunSizeMB = [decimal]::round($lun.ManagedCapacity/1024/1024)
            $totalLunSizeMB += $lunSizeMB
            $lunCount++
         }

         Write-Host "Number of LUNs:            $lunCount"
         Write-Host "Managed Capacity Size:     $totalLunSizeMB (MB)`n"

      }
      catch
      {
         Write-Host $_.Exception
      }

      try
      {
         Remove-Module -Name FmModuleSql
         createXmlFile $dbCount $totalUsedDBSizeMB $totalDbCountBE $dbSizeMBBE $lunCount $totalLunSizeMB $offload
      }
      catch
      {
         Write-Host $_.Exception
      }
   }      
}
 
function Get-FcmSqlusedDbSize
{ 
   process 
   {
      try
      {
         $timeStamp = get-date
         $databases = Get-SqlDatabasePrivate -sqlInstance $applicationentity
      
         foreach ($db in $databases)
         {
            if ($db.LastBackupDate -ne "1/1/0001")
            {
               $dbParent   =  [String]$db.Parent
               $dbParent   = $dbParent.Substring(1,$dbParent.Length - 2)
               $properties = @{
                  'ComputerName'=(hostname);
                  'InstanceName'=$dbParent;
                  'DbName'=$db.Name;
                  'AAGName' =$db.AvailabilityGroupName;
                  'UsedSizeKb'=$db.DataSpaceUsage + $db.IndexSpaceUsage;
                  'LastBackupDate'=$db.LastBackupDate;
                  'SampleDate'=$timeStamp;
               }

               $obj = New-Object -TypeName PSObject -Property $properties 
               write-output $obj | select ComputerName,InstanceName,DbName,AAGName,UsedSizeKb,LastBackupDate,SampleDate  
            }
         }
      }   
      catch
      {
         Write-Host $_.Exception
      }
   }      
} 


function Get-SqlDatabasePrivate
{  
   process 
   {
      try
      {
         $srv = New-Object ("Microsoft.SqlServer.Management.Smo.Server") $applicationentity
      
         if (0 -eq $dbName.count)
         { $srv.Databases }
         else
         { 
            foreach ($db in $dbName)
            {
               if ($db.Contains("*"))
                  { $srv.databases | where {$_.name -like $db} }
               else
               { $srv.databases | where {$_.name -eq $db} } 
            }
         }
      }
      catch
      {
         Write-Host $_.Exception
      }
   }      
}

function Get-FcmSqlManagedCapacity()
{
   Write-Host "`nQuerying Managed Capacity...`n"
   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOut = Invoke-Command -Command{Get-DpSqlManagedCapacity -detail -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOut = Invoke-Command -Command{Get-DpSqlManagedCapacity -detail}
   }
    
   
   return $cmdOut
}

function Get-FcmSqlBackups()
{
   Write-Host "`nQuerying SQL Backups...`n"

   if (($mycfg) -and ($myopt))
   {
      $cmdOut   = @(Invoke-Command -Command{Get-DpSqlBackup -All -full -FROMSQLServer * -QueryNode Both  -configFile $configfile -TsmOptFile $tsmoptfile})
      $cmdOut  += @(Invoke-Command -Command{Get-DpSqlBackup -All -copyfull -FROMSQLServer * -QueryNode Both  -configFile $configfile -TsmOptFile $tsmoptfile})
   }
   else
   {
      $cmdOut   = @(Invoke-Command -Command{Get-DpSqlBackup -All -full -FROMSQLServer * -QueryNode Both })
      $cmdOut  += @(Invoke-Command -Command{Get-DpSqlBackup -All -copyfull -FROMSQLServer * -QueryNode Both})
   }
   
   return $cmdOut
}


function createXmlFile($dbCount, $dbSizeFE, $countBE, $dbSizeBE, $lunCount, $lunSize, $offload)
{ 
   $cmdStr = '.\dsmfecc "--create" "--namespace=$namespace" "--productid=$productId" "--directory=$directory" "--applicationentity=$namespace" "--numberofobjects=$dbCount" "--size=$dbSizeFE" "--type=$itemTtype" "--fcmbenumberofobjects=$countBE" "--fcmbesize=$dbSizeBE" "--fcmlunnumberofobjects=$lunCount" "--fcmlunsize=$lunSize" "--offload=$offload" 2>&1'
   
   Write-Host "`nCreate XML file"
      
   Invoke-Expression $cmdStr
   if ($LastExitCode -ne 0)
   {
      Write-Host "Create XML file failed."      
      exit
   }    
}


Write-Host "`n"
Write-Host "********************************************************************"
Write-Host "********      IBM Spectrum Protect Suite - Front End       *********"
Write-Host "********          Terabyte (TB) Capacity Report            *********"
Write-Host "********************************************************************`n"
Write-Host "$productName`n"

Show-FcmSqlReport
