﻿param(
[Parameter(Mandatory=$True,
           helpmessage="Unique name to ensure data is counted only once and result can be identifed")]
           [string]$namespace,

[Parameter(Mandatory=$True,
           helpmessage="Enter directory where the fcm instance is located")]
           [string]$fcminstance, 

[Parameter(Mandatory=$True,
           helpmessage="Enter directory where the output should be written to")]
           [string]$directory,

[Parameter(Mandatory=$False,
           helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot options file")]
           [string]
           $tsmoptfile,

[Parameter(Mandatory=$False,
           helpmessage="Enter absolute path to IBM Spectrum Protect Snapshot configurations file")]
           [string]
           $configfile,

[Parameter(Mandatory=$False,
           helpmessage="Enter the BE Factor 0-1")]
           [decimal]$befactor          
)

$productId   = "18"
$productName = "IBM Spectrum Protect Snapshot for Custom Applications"
$itemTtype   = "fcm";

function parseUsageData($cmdOutFE, $cmdOutLU, $cmdOutBE)
{

   $mountSizeFE = 0
   $backupCount = 0
   $mountSizeLU = 0
   $countLu = 0
   $mountSizeBE = 0
   $countBE = 0
   $offload = 'n'
   $offloadFlag = $false

    
   if($befactor -le 0 -or $befactor -ge 1)
   {
      $befactor = 1;
   }

   if (($configfile) -and ($tsmoptfile))
   {
      $component = Invoke-Command -Command{Get-FcmFsComponent -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $component = Invoke-Command -Command{Get-FcmFsComponent}
   }

   $backupFE = $cmdOutFE.VolumeMountPoint | Sort-Object | Get-Unique
   
   $backupCount = $cmdOutFE | Measure-Object
   $backupCount = $backupCount.Count

   foreach ($mountFE in $backupFE)
   {
      foreach ($mount in $component) 
      {
         $mountName = $mount.VolumeMountPoint
         if ($mountFE.equals($mountName))
         {
             Write-Host "`nGetting front end size for:" $mountFE " Size(MB):" ([decimal]::round($mount.VolumeOccupancyGB * 1024))
             $mountSizeFE += ($mount.VolumeOccupancyGB*1024)
            
         }
      }
   }

   foreach ($backupLU in $cmdOutLU)
   {
      [int]$sizeLU = [decimal]::round($backupLU.ManagedCapacity/1024/1024)
      $mountSizeLU += $sizeLU
      $countLU++
   }

    foreach ($backupBE in $cmdOutBE)
   {
      if ($backupBE.BackupLocation.equals('Loc'))
       {
          $countBE++ 
       }
   }

    foreach ($backupLocation in $cmdOutBE)
   {
      if ($backupLocation.BackupLocation.equals('Srv'))
      {
         $offload = 'y'
         $offloadFlag = $True
         break;
      }
   }

    foreach ($mountFE in $backupFE)
   {
      foreach ($mount in $component)
      {
         $mountName = $mount.VolumeMountPoint
         if ($mountFE.equals($mountName))
         {
             $sizeBE = ($mount.VolumeOccupancyGB*1024) * $countBE * $befactor
             Write-Host "`nGetting back end size for:" $mountFE " Size(MB):" ([decimal]::round($sizeBE))
             $mountSizeBE += ($sizeBE)
         }
      }
   }
   
   
   Remove-Module -Name FmModuleFS

   $mountSizeFE = [decimal]::round($mountSizeFE)
   $mountSizeBE = [decimal]::round($mountSizeBE)
   

   Write-Host "`nIBM Spectrum Protect Snapshot FE"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-Host "Number of objects:              $backupCount"
   Write-Host "Custom Application (fcm fe):    $mountSizeFE (MB)"  

   Write-Host "`nIBM Spectrum Protect Snapshot LUN"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-host "Number of objects:              $countLU"
   Write-Host "Custom Application (fcm lun):   $mountSizeLU (MB)" 

   Write-Host "`nIBM Spectrum Protect Snapshot BE"
   Write-Host "------------------------------------------"
   Write-Host "Query result"
   Write-Host "Snapshots:                      $countBE" 
   Write-Host "Custom Application (fcm be):    $mountSizeBE (MB)`n" 

   if ($offloadFlag)
   {
      Write-Host "Backups to IBM Spectrum Protect Server detected: Yes!`n"
   }
   else
   {
      Write-Host "Backups to IBM Spectrum Protect Server detected: No`n"
   }
   
   createXmlFile $backupCount $mountSizeFE $countBE $mountSizeBE $countLU $mountSizeLU $offload
        
}

function fcmFE()
{
   $cmdStrLU = $fcminstance.Substring(0,$fcminstance.Length)
   $cmdStrLU = $cmdStrLU + "\FmModuleFs.dll"
   Import-Module $cmdStrLU
   
   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOutFE = Invoke-Command -Command{Get-FcmFsBackup -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOutFE = Invoke-Command -Command{Get-FcmFsBackup}
   }

   return $cmdOutFE   
}

function fcmLU()
{
   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOutLU = Invoke-Command -Command{Get-FcmFsManagedCapacity -detail -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOutLU = Invoke-Command -Command{Get-FcmFsManagedCapacity -detail}
   }

   return $cmdOutLU
}

function fcmBE()
{
   $commandString  = ""

   if (($configfile) -and ($tsmoptfile))
   {
      $cmdOutBE = Invoke-Command -Command{Get-FcmFsBackup -all -configFile $configfile -TsmOptFile $tsmoptfile}
   }
   else
   {
      $cmdOutBE = Invoke-Command -Command{Get-FcmFsBackup -all}
   }
   
   return $cmdOutBE
}


function createXmlFile($backupCount, $mountSizeFE, $countBE, $mountSizeBE, $countLU, $mountSizeLU, $offload)
{ 
   $cmdStr = '.\dsmfecc "--create" "--namespace=$namespace" "--productid=$productId" "--directory=$directory" "--applicationentity=$namespace" "--numberofobjects=$backupCount" "--size=$mountSizeFE" "--type=$itemTtype" "--fcmbenumberofobjects=$countBE" "--fcmbesize=$mountSizeBE" "--fcmlunnumberofobjects=$countLU" "--fcmlunsize=$mountSizeLU" "--offload=$offload" 2>&1'
   
   Write-Host "`nCreate XML file"
      
   Invoke-Expression $cmdStr
   if ($LastExitCode -ne 0)
   {
      Write-Host "Create XML file failed."      
      exit
   }    
}

Write-Host "`n"
Write-Host "********************************************************************"
Write-Host "********      IBM Spectrum Protect Suite - Front End       *********"
Write-Host "********          Terabyte (TB) Capacity Report            *********"
Write-Host "********************************************************************`n"
Write-Host "$productName`n"

$queryFE = fcmFE
$queryLU = fcmLU
$queryBE = fcmBE
parseUsageData $queryFE $queryLU $queryBE


